# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# by BJ Dierkes <wdierkes@rackspace.com> 

'''
This plugin is intended to drop stock packages from the package list, only 
if a package that provides it or its base package is installed.  Reason
in doing so is in the case of packages with different names that provide 
the same thing.

For example:

    repo1 has php53, php53-mysql, php53-pdo, etc
    stock rhel has php, php-mysql, php-pdo, etc

If just php53 is installed, and you attempt to install something that
requires "php-mysql", even though php53-mysql provides php-mysql Yum will
attempt to install php-mysql. 
'''

import sys
from yum.plugins import TYPE_CORE
from yum import config 

requires_api_version = '2.4'
plugin_type = (TYPE_CORE,)

def config_hook(conduit):
    '''Add options to Yum's configuration
    '''
    config.YumConf.autoexclude = config.BoolOption(False)
    config.RepoConf.autoexclude = config.Inherit(config.YumConf.autoexclude)
    pass

def exclude_hook(conduit):
    '''
    Automatically exclude packages from other repositories if a package or
    it's base package is installed and provides that package name.
    '''
    cnt = 0

    allrepos = conduit.getRepos().listEnabled()

    for repo1 in allrepos:
        if repo1.enabled and repo1.autoexclude:
            rpmdb = conduit.getRpmDB()
            repo1pkgs = conduit.getPackages(repo1)
            ius_base_installed = {} 
            for po in repo1pkgs:
                # find base package, if installed
                pkg_split = po.name.split('-')
                pkg_base = None 
                pkg_base_installed = False
                for i in pkg_split:
                    if not pkg_base:
                        pkg_base = i
                    else:
                        pkg_base = "%s-%s" % (pkg_base, i)
                    if rpmdb.installed(name=pkg_base):
                        pkg_base_installed = True
                        break

                if not pkg_base_installed:
                    continue

                conduit.info(9, "Base package %s is installed, processing excludes for %s" % (
                    pkg_base, po.name
                    ))
                provides = _provides_to_tuple(po.provides_names)

                for repo2 in allrepos:
                    if not repo2.enabled:
                        continue

                    for po2 in conduit.getPackages(repo2):
                        if po2.name in provides and po2.name != po.name:
                            conduit.delPackage(po2)
                            conduit.info(9, 
                                "Excluding %s-%s-%s to avoid conflict with %s-%s-%s" % (
                                    po2.name, po2.version, po2.release, po.name, po.version, po.release
                                    )
                                )
                            cnt += 1

    conduit.info(2, '%d packages excluded by autoexclude, due to base packages being installed' % cnt)

def _provides_to_tuple(pp):
    out = [] 
    for p in pp:
        out.append(p)
    return out

